# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility functions for normalizing gRPC messages."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import datetime
import sys

from cloudsdk.google.protobuf import json_format
from googlecloudsdk.api_lib.storage import metadata_util
from googlecloudsdk.api_lib.storage.gcs_json import metadata_util as json_metadata_util
from googlecloudsdk.command_lib.storage import hash_util
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.command_lib.storage.resources import gcs_resource_reference
from googlecloudsdk.command_lib.util import crc32c

# pylint:disable=g-import-not-at-top
try:
  # TODO(b/277356731) Remove version check after gcloud drops Python <= 3.5.
  if sys.version_info.major == 3 and sys.version_info.minor > 5:
    from google.api_core.gapic_v1 import routing_header
except ImportError:
  pass
# pylint:enable=g-import-not-at-top


GRPC_URL_BUCKET_OFFSET = len('projects/_/buckets/')


def _convert_repeated_message_to_dict(message):
  """Converts a sequence of proto messages to a dict."""
  if not message:
    return
  # TODO(b/262768337) Use message_to_dict translation once it's fixed
  # pylint: disable=protected-access
  return [json_format.MessageToDict(i._pb) for i in message]
  # pylint: enable=protected-access


def _convert_proto_to_datetime(proto_datetime):
  """Converts the proto.datetime_helpers.DatetimeWithNanoseconds to datetime."""
  if not proto_datetime:
    return
  return datetime.datetime.fromtimestamp(
      proto_datetime.timestamp(), proto_datetime.tzinfo)


def _get_value_or_none(value):
  """Returns None if value is falsy, else the value itself.

  Unlike Apitools messages, gRPC messages do not return None for fields that
  are not set. It will instead be set to a falsy value.

  Args:
    value (proto.Message): The proto message.

  Returns:
    None if the value is falsy, else the value itself.
  """
  if value:
    return value
  return None


def get_object_resource_from_grpc_object(grpc_object):
  """Returns the GCSObjectResource based off of the gRPC Object."""
  if grpc_object.generation is not None:
    # Generation may be 0 integer, which is valid although falsy.
    generation = str(grpc_object.generation)
  else:
    generation = None
  url = storage_url.CloudUrl(
      scheme=storage_url.ProviderPrefix.GCS,
      # bucket is of the form projects/_/buckets/<bucket_name>
      bucket_name=grpc_object.bucket[GRPC_URL_BUCKET_OFFSET:],
      object_name=grpc_object.name,
      generation=generation)

  if (grpc_object.customer_encryption and
      grpc_object.customer_encryption.key_sha256_bytes):
    decryption_key_hash_sha256 = hash_util.get_base64_string(
        grpc_object.customer_encryption.key_sha256_bytes)
    encryption_algorithm = grpc_object.customer_encryption.encryption_algorithm
  else:
    decryption_key_hash_sha256 = encryption_algorithm = None

  if grpc_object.checksums.crc32c is not None:
    # crc32c can be 0, so check for None value specifically.
    crc32c_hash = crc32c.get_crc32c_hash_string_from_checksum(
        grpc_object.checksums.crc32c)
  else:
    crc32c_hash = None

  if grpc_object.checksums.md5_hash:
    md5_hash = hash_util.get_base64_string(grpc_object.checksums.md5_hash)
  else:
    md5_hash = None

  return gcs_resource_reference.GcsObjectResource(
      url,
      acl=_convert_repeated_message_to_dict(grpc_object.acl),
      cache_control=_get_value_or_none(grpc_object.cache_control),
      component_count=_get_value_or_none(grpc_object.component_count),
      content_disposition=_get_value_or_none(grpc_object.content_disposition),
      content_encoding=_get_value_or_none(grpc_object.content_encoding),
      content_language=_get_value_or_none(grpc_object.content_language),
      content_type=_get_value_or_none(grpc_object.content_type),
      crc32c_hash=crc32c_hash,
      creation_time=_convert_proto_to_datetime(grpc_object.create_time),
      custom_fields=_get_value_or_none(grpc_object.metadata),
      custom_time=_convert_proto_to_datetime(grpc_object.custom_time),
      decryption_key_hash_sha256=decryption_key_hash_sha256,
      encryption_algorithm=encryption_algorithm,
      etag=_get_value_or_none(grpc_object.etag),
      event_based_hold=(grpc_object.event_based_hold
                        if grpc_object.event_based_hold else None),
      kms_key=_get_value_or_none(grpc_object.kms_key),
      md5_hash=md5_hash,
      metadata=grpc_object,
      metageneration=grpc_object.metageneration,
      noncurrent_time=_convert_proto_to_datetime(grpc_object.delete_time),
      retention_expiration=_convert_proto_to_datetime(
          grpc_object.retention_expire_time),
      size=grpc_object.size,
      storage_class=_get_value_or_none(grpc_object.storage_class),
      storage_class_update_time=_convert_proto_to_datetime(
          grpc_object.update_storage_class_time),
      temporary_hold=(grpc_object.temporary_hold
                      if grpc_object.temporary_hold else None),
      update_time=_convert_proto_to_datetime(grpc_object.update_time))


def update_object_metadata_from_request_config(
    object_metadata, request_config, attributes_resource=None
):
  # pylint: disable=line-too-long
  """Sets GRPC Storage Object fields based on values in request config.

  Checksums such as md5 are not set because they are ignored if they are
  provided.

  Args:
    object_metadata (gapic_clients.storage_v2.types.storage.Object): Existing
      object metadata.
    request_config (request_config_factory._GcsRequestConfig): May contain data
      to add to object_metadata.
    attributes_resource (resource_reference.FileObjectResource|resource_reference.ObjectResource|None):
      Contains the source StorageUrl and source object metadata for daisy chain
      transfers. Can be None if source is pure stream
  """
  # pylint: enable=line-too-long
  resource_args = request_config.resource_args

  custom_fields_dict = metadata_util.get_updated_custom_fields(
      object_metadata.metadata,
      request_config,
      attributes_resource
  )

  json_metadata_util.process_value_or_clear_flag(
      object_metadata, 'metadata', custom_fields_dict
  )

  should_gzip_locally = json_metadata_util.get_should_gzip_locally(
      attributes_resource, request_config
  )

  content_encoding = json_metadata_util.get_content_encoding(
      should_gzip_locally, resource_args
  )
  cache_control = json_metadata_util.get_cache_control(
      should_gzip_locally, resource_args
  )

  json_metadata_util.process_value_or_clear_flag(
      object_metadata, 'cache_control', cache_control
  )
  json_metadata_util.process_value_or_clear_flag(
      object_metadata, 'content_encoding', content_encoding
  )

  if not resource_args:
    return

  json_metadata_util.process_value_or_clear_flag(
      object_metadata, 'content_disposition', resource_args.content_disposition
  )
  json_metadata_util.process_value_or_clear_flag(
      object_metadata, 'content_language', resource_args.content_language
  )
  json_metadata_util.process_value_or_clear_flag(
      object_metadata, 'content_type', resource_args.content_type
  )
  json_metadata_util.process_value_or_clear_flag(
      object_metadata, 'custom_time', resource_args.custom_time
  )


def get_bucket_name_routing_header(bucket_name):
  """Gets routing header with bucket.

  Args:
    bucket_name (str): Name of the bucket.

  Returns:
    (List[Tuple[str, str]]) List with metadata.
  """
  return [routing_header.to_grpc_metadata({'bucket': bucket_name})]



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Create a new ekm connection."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.kms import certs
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.command_lib.kms import maps
from googlecloudsdk.command_lib.kms import resource_args


class Create(base.CreateCommand):
  r"""Create a new ekm connection.

  Creates a new connection within the given location.

  ## EXAMPLES

  The following command creates an ekm connection named `laplace` within the
  location `us-central1`:

    $ {command} laplace \
        --location=us-central1 \
        --service-directory-service="foo" \
        --endpoint-filter="foo > bar" \
        --hostname="hostname.foo" \
        --server-certificates-files=foo.pem,bar.pem

  The following command creates an ekm connection named `laplace` within the
  location `us-central1` in `cloud-kms` key management mode with the required
  crypto-space-path :

    $ {command} laplace \
        --location=us-central1 \
        --service-directory-service="foo" \
        --endpoint-filter="foo > bar" \
        --hostname="hostname.foo" \
        --key-management-mode=cloud-kms
        --crypto-space-path="foo"
        --server-certificates-files=foo.pem,bar.pem
  """

  @staticmethod
  def Args(parser):
    resource_args.AddKmsEkmConnectionResourceArgForKMS(parser, True,
                                                       'ekm_connection')
    flags.AddServiceDirectoryServiceFlag(parser, True)
    flags.AddEndpointFilterFlag(parser)
    flags.AddHostnameFlag(parser, True)
    flags.AddServerCertificatesFilesFlag(parser, True)
    flags.AddKeyManagementModeFlags(parser)
    parser.display_info.AddCacheUpdater(flags.EkmConnectionCompleter)

  def _CreateRequest(self, args):
    messages = cloudkms_base.GetMessagesModule()

    ekm_connection_ref = args.CONCEPTS.ekm_connection.Parse()
    parent_ref = ekm_connection_ref.Parent()

    if args.key_management_mode == 'cloud-kms':
      if not args.crypto_space_path:
        raise exceptions.RequiredArgumentException(
            '--crypto-space-path',
            'Must be supplied when --key-management-mode is cloud-kms.')

    certificate_list = []
    for cert_file in args.server_certificates_files:
      try:
        certificate_list.append(
            messages.Certificate(rawDer=certs.GetDerCertificate(cert_file)))
      except Exception as e:
        raise exceptions.BadArgumentException(
            '--server-certificates-files',
            'Error while attempting to read file {} : {}'.format(cert_file, e))

    req = messages.CloudkmsProjectsLocationsEkmConnectionsCreateRequest(
        parent=parent_ref.RelativeName(),
        ekmConnectionId=ekm_connection_ref.Name(),
        ekmConnection=messages.EkmConnection(
            keyManagementMode=maps.KEY_MANAGEMENT_MODE_MAPPER.GetEnumForChoice(
                args.key_management_mode),
            cryptoSpacePath=args.crypto_space_path,
            serviceResolvers=[
                messages.ServiceResolver(
                    serviceDirectoryService=args.service_directory_service,
                    endpointFilter=args.endpoint_filter,
                    hostname=args.hostname,
                    serverCertificates=certificate_list)
            ]))

    return req

  def Run(self, args):
    client = cloudkms_base.GetClientInstance()
    return client.projects_locations_ekmConnections.Create(
        self._CreateRequest(args))



"""Generated client library for connectors version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.connectors.v1 import connectors_v1_messages as messages


class ConnectorsV1(base_api.BaseApiClient):
  """Generated client library for service connectors version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://connectors.googleapis.com/'
  MTLS_BASE_URL = 'https://connectors.mtls.googleapis.com/'

  _PACKAGE = 'connectors'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'ConnectorsV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new connectors handle."""
    url = url or self.BASE_URL
    super(ConnectorsV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_connections_connectionSchemaMetadata = self.ProjectsLocationsConnectionsConnectionSchemaMetadataService(self)
    self.projects_locations_connections_eventSubscriptions = self.ProjectsLocationsConnectionsEventSubscriptionsService(self)
    self.projects_locations_connections_runtimeActionSchemas = self.ProjectsLocationsConnectionsRuntimeActionSchemasService(self)
    self.projects_locations_connections_runtimeEntitySchemas = self.ProjectsLocationsConnectionsRuntimeEntitySchemasService(self)
    self.projects_locations_connections = self.ProjectsLocationsConnectionsService(self)
    self.projects_locations_endpointAttachments = self.ProjectsLocationsEndpointAttachmentsService(self)
    self.projects_locations_global_managedZones = self.ProjectsLocationsGlobalManagedZonesService(self)
    self.projects_locations_global = self.ProjectsLocationsGlobalService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_providers_connectors_versions_eventtypes = self.ProjectsLocationsProvidersConnectorsVersionsEventtypesService(self)
    self.projects_locations_providers_connectors_versions = self.ProjectsLocationsProvidersConnectorsVersionsService(self)
    self.projects_locations_providers_connectors = self.ProjectsLocationsProvidersConnectorsService(self)
    self.projects_locations_providers = self.ProjectsLocationsProvidersService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsConnectionsConnectionSchemaMetadataService(base_api.BaseApiService):
    """Service class for the projects_locations_connections_connectionSchemaMetadata resource."""

    _NAME = 'projects_locations_connections_connectionSchemaMetadata'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsConnectionsConnectionSchemaMetadataService, self).__init__(client)
      self._upload_configs = {
          }

    def Refresh(self, request, global_params=None):
      r"""Refresh runtime schema of a connection.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsConnectionSchemaMetadataRefreshRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Refresh')
      return self._RunMethod(
          config, request, global_params=global_params)

    Refresh.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/connectionSchemaMetadata:refresh',
        http_method='POST',
        method_id='connectors.projects.locations.connections.connectionSchemaMetadata.refresh',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:refresh',
        request_field='refreshConnectionSchemaMetadataRequest',
        request_type_name='ConnectorsProjectsLocationsConnectionsConnectionSchemaMetadataRefreshRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsConnectionsEventSubscriptionsService(base_api.BaseApiService):
    """Service class for the projects_locations_connections_eventSubscriptions resource."""

    _NAME = 'projects_locations_connections_eventSubscriptions'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsConnectionsEventSubscriptionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new EventSubscription in a given project,location and connection.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsEventSubscriptionsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/eventSubscriptions',
        http_method='POST',
        method_id='connectors.projects.locations.connections.eventSubscriptions.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['eventSubscriptionId'],
        relative_path='v1/{+parent}/eventSubscriptions',
        request_field='eventSubscription',
        request_type_name='ConnectorsProjectsLocationsConnectionsEventSubscriptionsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single EventSubscription.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsEventSubscriptionsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/eventSubscriptions/{eventSubscriptionsId}',
        http_method='DELETE',
        method_id='connectors.projects.locations.connections.eventSubscriptions.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsEventSubscriptionsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single EventSubscription.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsEventSubscriptionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventSubscription) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/eventSubscriptions/{eventSubscriptionsId}',
        http_method='GET',
        method_id='connectors.projects.locations.connections.eventSubscriptions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsEventSubscriptionsGetRequest',
        response_type_name='EventSubscription',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List EventSubscriptions in a given project,location and connection.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsEventSubscriptionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListEventSubscriptionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/eventSubscriptions',
        http_method='GET',
        method_id='connectors.projects.locations.connections.eventSubscriptions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/eventSubscriptions',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsEventSubscriptionsListRequest',
        response_type_name='ListEventSubscriptionsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single EventSubscription.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsEventSubscriptionsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/eventSubscriptions/{eventSubscriptionsId}',
        http_method='PATCH',
        method_id='connectors.projects.locations.connections.eventSubscriptions.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='eventSubscription',
        request_type_name='ConnectorsProjectsLocationsConnectionsEventSubscriptionsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Retry(self, request, global_params=None):
      r"""RetryEventSubscription retries the registration of Subscription.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsEventSubscriptionsRetryRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Retry')
      return self._RunMethod(
          config, request, global_params=global_params)

    Retry.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/eventSubscriptions/{eventSubscriptionsId}:retry',
        http_method='POST',
        method_id='connectors.projects.locations.connections.eventSubscriptions.retry',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:retry',
        request_field='retryEventSubscriptionRequest',
        request_type_name='ConnectorsProjectsLocationsConnectionsEventSubscriptionsRetryRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsConnectionsRuntimeActionSchemasService(base_api.BaseApiService):
    """Service class for the projects_locations_connections_runtimeActionSchemas resource."""

    _NAME = 'projects_locations_connections_runtimeActionSchemas'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsConnectionsRuntimeActionSchemasService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""List schema of a runtime actions filtered by action name.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsRuntimeActionSchemasListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListRuntimeActionSchemasResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/runtimeActionSchemas',
        http_method='GET',
        method_id='connectors.projects.locations.connections.runtimeActionSchemas.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/runtimeActionSchemas',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsRuntimeActionSchemasListRequest',
        response_type_name='ListRuntimeActionSchemasResponse',
        supports_download=False,
    )

  class ProjectsLocationsConnectionsRuntimeEntitySchemasService(base_api.BaseApiService):
    """Service class for the projects_locations_connections_runtimeEntitySchemas resource."""

    _NAME = 'projects_locations_connections_runtimeEntitySchemas'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsConnectionsRuntimeEntitySchemasService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""List schema of a runtime entities filtered by entity name.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsRuntimeEntitySchemasListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListRuntimeEntitySchemasResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/runtimeEntitySchemas',
        http_method='GET',
        method_id='connectors.projects.locations.connections.runtimeEntitySchemas.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/runtimeEntitySchemas',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsRuntimeEntitySchemasListRequest',
        response_type_name='ListRuntimeEntitySchemasResponse',
        supports_download=False,
    )

  class ProjectsLocationsConnectionsService(base_api.BaseApiService):
    """Service class for the projects_locations_connections resource."""

    _NAME = 'projects_locations_connections'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsConnectionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new Connection in a given project and location.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections',
        http_method='POST',
        method_id='connectors.projects.locations.connections.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['connectionId'],
        relative_path='v1/{+parent}/connections',
        request_field='connection',
        request_type_name='ConnectorsProjectsLocationsConnectionsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single Connection.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}',
        http_method='DELETE',
        method_id='connectors.projects.locations.connections.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single Connection.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Connection) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}',
        http_method='GET',
        method_id='connectors.projects.locations.connections.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['view'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsGetRequest',
        response_type_name='Connection',
        supports_download=False,
    )

    def GetConnectionSchemaMetadata(self, request, global_params=None):
      r"""Gets schema metadata of a connection. SchemaMetadata is a singleton resource for each connection.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsGetConnectionSchemaMetadataRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ConnectionSchemaMetadata) The response message.
      """
      config = self.GetMethodConfig('GetConnectionSchemaMetadata')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetConnectionSchemaMetadata.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}/connectionSchemaMetadata',
        http_method='GET',
        method_id='connectors.projects.locations.connections.getConnectionSchemaMetadata',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsGetConnectionSchemaMetadataRequest',
        response_type_name='ConnectionSchemaMetadata',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}:getIamPolicy',
        http_method='GET',
        method_id='connectors.projects.locations.connections.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Connections in a given project and location.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListConnectionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections',
        http_method='GET',
        method_id='connectors.projects.locations.connections.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken', 'view'],
        relative_path='v1/{+parent}/connections',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsConnectionsListRequest',
        response_type_name='ListConnectionsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single Connection.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}',
        http_method='PATCH',
        method_id='connectors.projects.locations.connections.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='connection',
        request_type_name='ConnectorsProjectsLocationsConnectionsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def RepairEventing(self, request, global_params=None):
      r"""RepaiEventing tries to repair eventing related event subscriptions.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsRepairEventingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RepairEventing')
      return self._RunMethod(
          config, request, global_params=global_params)

    RepairEventing.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}:repairEventing',
        http_method='POST',
        method_id='connectors.projects.locations.connections.repairEventing',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:repairEventing',
        request_field='repairEventingRequest',
        request_type_name='ConnectorsProjectsLocationsConnectionsRepairEventingRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy. Can return `NOT_FOUND`, `INVALID_ARGUMENT`, and `PERMISSION_DENIED` errors.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}:setIamPolicy',
        http_method='POST',
        method_id='connectors.projects.locations.connections.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='ConnectorsProjectsLocationsConnectionsSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a `NOT_FOUND` error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (ConnectorsProjectsLocationsConnectionsTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/connections/{connectionsId}:testIamPermissions',
        http_method='POST',
        method_id='connectors.projects.locations.connections.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='ConnectorsProjectsLocationsConnectionsTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsEndpointAttachmentsService(base_api.BaseApiService):
    """Service class for the projects_locations_endpointAttachments resource."""

    _NAME = 'projects_locations_endpointAttachments'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsEndpointAttachmentsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new EndpointAttachment in a given project and location.

      Args:
        request: (ConnectorsProjectsLocationsEndpointAttachmentsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/endpointAttachments',
        http_method='POST',
        method_id='connectors.projects.locations.endpointAttachments.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['endpointAttachmentId'],
        relative_path='v1/{+parent}/endpointAttachments',
        request_field='endpointAttachment',
        request_type_name='ConnectorsProjectsLocationsEndpointAttachmentsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single EndpointAttachment.

      Args:
        request: (ConnectorsProjectsLocationsEndpointAttachmentsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/endpointAttachments/{endpointAttachmentsId}',
        http_method='DELETE',
        method_id='connectors.projects.locations.endpointAttachments.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsEndpointAttachmentsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single EndpointAttachment.

      Args:
        request: (ConnectorsProjectsLocationsEndpointAttachmentsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EndpointAttachment) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/endpointAttachments/{endpointAttachmentsId}',
        http_method='GET',
        method_id='connectors.projects.locations.endpointAttachments.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsEndpointAttachmentsGetRequest',
        response_type_name='EndpointAttachment',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List EndpointAttachments in a given project.

      Args:
        request: (ConnectorsProjectsLocationsEndpointAttachmentsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListEndpointAttachmentsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/endpointAttachments',
        http_method='GET',
        method_id='connectors.projects.locations.endpointAttachments.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/endpointAttachments',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsEndpointAttachmentsListRequest',
        response_type_name='ListEndpointAttachmentsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single EndpointAttachment.

      Args:
        request: (ConnectorsProjectsLocationsEndpointAttachmentsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/endpointAttachments/{endpointAttachmentsId}',
        http_method='PATCH',
        method_id='connectors.projects.locations.endpointAttachments.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='endpointAttachment',
        request_type_name='ConnectorsProjectsLocationsEndpointAttachmentsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsGlobalManagedZonesService(base_api.BaseApiService):
    """Service class for the projects_locations_global_managedZones resource."""

    _NAME = 'projects_locations_global_managedZones'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsGlobalManagedZonesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new ManagedZone in a given project and location.

      Args:
        request: (ConnectorsProjectsLocationsGlobalManagedZonesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/global/managedZones',
        http_method='POST',
        method_id='connectors.projects.locations.global.managedZones.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['managedZoneId'],
        relative_path='v1/{+parent}/managedZones',
        request_field='managedZone',
        request_type_name='ConnectorsProjectsLocationsGlobalManagedZonesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single ManagedZone.

      Args:
        request: (ConnectorsProjectsLocationsGlobalManagedZonesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/global/managedZones/{managedZonesId}',
        http_method='DELETE',
        method_id='connectors.projects.locations.global.managedZones.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsGlobalManagedZonesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single ManagedZone.

      Args:
        request: (ConnectorsProjectsLocationsGlobalManagedZonesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ManagedZone) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/global/managedZones/{managedZonesId}',
        http_method='GET',
        method_id='connectors.projects.locations.global.managedZones.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsGlobalManagedZonesGetRequest',
        response_type_name='ManagedZone',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List ManagedZones in a given project.

      Args:
        request: (ConnectorsProjectsLocationsGlobalManagedZonesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListManagedZonesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/global/managedZones',
        http_method='GET',
        method_id='connectors.projects.locations.global.managedZones.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/managedZones',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsGlobalManagedZonesListRequest',
        response_type_name='ListManagedZonesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single ManagedZone.

      Args:
        request: (ConnectorsProjectsLocationsGlobalManagedZonesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/global/managedZones/{managedZonesId}',
        http_method='PATCH',
        method_id='connectors.projects.locations.global.managedZones.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='managedZone',
        request_type_name='ConnectorsProjectsLocationsGlobalManagedZonesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsGlobalService(base_api.BaseApiService):
    """Service class for the projects_locations_global resource."""

    _NAME = 'projects_locations_global'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsGlobalService, self).__init__(client)
      self._upload_configs = {
          }

    def GetSettings(self, request, global_params=None):
      r"""GetGlobalSettings gets settings of a project. GlobalSettings is a singleton resource.

      Args:
        request: (ConnectorsProjectsLocationsGlobalGetSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Settings) The response message.
      """
      config = self.GetMethodConfig('GetSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/global/settings',
        http_method='GET',
        method_id='connectors.projects.locations.global.getSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsGlobalGetSettingsRequest',
        response_type_name='Settings',
        supports_download=False,
    )

    def UpdateSettings(self, request, global_params=None):
      r"""Update the global settings of a project.

      Args:
        request: (ConnectorsProjectsLocationsGlobalUpdateSettingsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('UpdateSettings')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateSettings.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/global/settings',
        http_method='PATCH',
        method_id='connectors.projects.locations.global.updateSettings',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='settings',
        request_type_name='ConnectorsProjectsLocationsGlobalUpdateSettingsRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.

      Args:
        request: (ConnectorsProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='connectors.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='ConnectorsProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (ConnectorsProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='connectors.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (ConnectorsProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='connectors.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (ConnectorsProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='connectors.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+name}/operations',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsProvidersConnectorsVersionsEventtypesService(base_api.BaseApiService):
    """Service class for the projects_locations_providers_connectors_versions_eventtypes resource."""

    _NAME = 'projects_locations_providers_connectors_versions_eventtypes'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsProvidersConnectorsVersionsEventtypesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets details of a single event type.

      Args:
        request: (ConnectorsProjectsLocationsProvidersConnectorsVersionsEventtypesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EventType) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}/connectors/{connectorsId}/versions/{versionsId}/eventtypes/{eventtypesId}',
        http_method='GET',
        method_id='connectors.projects.locations.providers.connectors.versions.eventtypes.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsProvidersConnectorsVersionsEventtypesGetRequest',
        response_type_name='EventType',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Event Types in a given Connector Version.

      Args:
        request: (ConnectorsProjectsLocationsProvidersConnectorsVersionsEventtypesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListEventTypesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}/connectors/{connectorsId}/versions/{versionsId}/eventtypes',
        http_method='GET',
        method_id='connectors.projects.locations.providers.connectors.versions.eventtypes.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/eventtypes',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsProvidersConnectorsVersionsEventtypesListRequest',
        response_type_name='ListEventTypesResponse',
        supports_download=False,
    )

  class ProjectsLocationsProvidersConnectorsVersionsService(base_api.BaseApiService):
    """Service class for the projects_locations_providers_connectors_versions resource."""

    _NAME = 'projects_locations_providers_connectors_versions'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsProvidersConnectorsVersionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets details of a single connector version.

      Args:
        request: (ConnectorsProjectsLocationsProvidersConnectorsVersionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ConnectorVersion) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}/connectors/{connectorsId}/versions/{versionsId}',
        http_method='GET',
        method_id='connectors.projects.locations.providers.connectors.versions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['view'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsProvidersConnectorsVersionsGetRequest',
        response_type_name='ConnectorVersion',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Connector Versions in a given project and location.

      Args:
        request: (ConnectorsProjectsLocationsProvidersConnectorsVersionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListConnectorVersionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}/connectors/{connectorsId}/versions',
        http_method='GET',
        method_id='connectors.projects.locations.providers.connectors.versions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken', 'view'],
        relative_path='v1/{+parent}/versions',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsProvidersConnectorsVersionsListRequest',
        response_type_name='ListConnectorVersionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsProvidersConnectorsService(base_api.BaseApiService):
    """Service class for the projects_locations_providers_connectors resource."""

    _NAME = 'projects_locations_providers_connectors'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsProvidersConnectorsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets details of a single Connector.

      Args:
        request: (ConnectorsProjectsLocationsProvidersConnectorsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Connector) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}/connectors/{connectorsId}',
        http_method='GET',
        method_id='connectors.projects.locations.providers.connectors.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsProvidersConnectorsGetRequest',
        response_type_name='Connector',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Connectors in a given project and location.

      Args:
        request: (ConnectorsProjectsLocationsProvidersConnectorsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListConnectorsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}/connectors',
        http_method='GET',
        method_id='connectors.projects.locations.providers.connectors.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/connectors',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsProvidersConnectorsListRequest',
        response_type_name='ListConnectorsResponse',
        supports_download=False,
    )

  class ProjectsLocationsProvidersService(base_api.BaseApiService):
    """Service class for the projects_locations_providers resource."""

    _NAME = 'projects_locations_providers'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsProvidersService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets details of a provider.

      Args:
        request: (ConnectorsProjectsLocationsProvidersGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Provider) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}',
        http_method='GET',
        method_id='connectors.projects.locations.providers.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsProvidersGetRequest',
        response_type_name='Provider',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (ConnectorsProjectsLocationsProvidersGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}:getIamPolicy',
        http_method='GET',
        method_id='connectors.projects.locations.providers.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsProvidersGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Providers in a given project and location.

      Args:
        request: (ConnectorsProjectsLocationsProvidersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListProvidersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers',
        http_method='GET',
        method_id='connectors.projects.locations.providers.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/providers',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsProvidersListRequest',
        response_type_name='ListProvidersResponse',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy. Can return `NOT_FOUND`, `INVALID_ARGUMENT`, and `PERMISSION_DENIED` errors.

      Args:
        request: (ConnectorsProjectsLocationsProvidersSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}:setIamPolicy',
        http_method='POST',
        method_id='connectors.projects.locations.providers.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='ConnectorsProjectsLocationsProvidersSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a `NOT_FOUND` error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (ConnectorsProjectsLocationsProvidersTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/providers/{providersId}:testIamPermissions',
        http_method='POST',
        method_id='connectors.projects.locations.providers.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='ConnectorsProjectsLocationsProvidersTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (ConnectorsProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='connectors.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def GetRuntimeConfig(self, request, global_params=None):
      r"""Gets the runtimeConfig of a location. RuntimeConfig is a singleton resource for each location.

      Args:
        request: (ConnectorsProjectsLocationsGetRuntimeConfigRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RuntimeConfig) The response message.
      """
      config = self.GetMethodConfig('GetRuntimeConfig')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetRuntimeConfig.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/runtimeConfig',
        http_method='GET',
        method_id='connectors.projects.locations.getRuntimeConfig',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsGetRuntimeConfigRequest',
        response_type_name='RuntimeConfig',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service.

      Args:
        request: (ConnectorsProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations',
        http_method='GET',
        method_id='connectors.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+name}/locations',
        request_field='',
        request_type_name='ConnectorsProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(ConnectorsV1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cross-version utility classes and functions for gcloud functions commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import abc
import re

from googlecloudsdk.api_lib.functions.v1 import util as api_util_v1
from googlecloudsdk.api_lib.functions.v2 import client as client_v2
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_extensions
from googlecloudsdk.command_lib.functions import flags

import six  # pylint: disable=unused-import # Somehow Pylint complains :(


class FunctionResourceCommand(six.with_metaclass(abc.ABCMeta, base.Command)):
  """Mix-in for single function resource commands that work with both v1 or v2.

  Which version of the command to run is determined by the following precedence:
  1. Explicit setting via the --gen2/--no-gen2 flags or functions/gen2 property.
  2. The generation of the function if it exists.
  2. The v1 API by default.

  Subclasses should add the function resource arg and --gen2 flag.
  """

  def __init__(self, *args, **kwargs):
    super(FunctionResourceCommand, self).__init__(*args, **kwargs)
    self._v2_function = None

  @abc.abstractmethod
  def _RunV1(self, args):
    # type: (parser_extensions.Namespace) -> Any
    """Runs the command against the v1 API."""

  @abc.abstractmethod
  def _RunV2(self, args):
    # type: (parser_extensions.Namespace) -> Any
    """Runs the command against the v2 API."""

  @api_util_v1.CatchHTTPErrorRaiseHTTPException
  def Run(self, args):
    # type: (parser_extensions.Namespace) -> Any
    """Runs the command.

    Args:
      args: The arguments this command was invoked with.

    Returns:
      The result of the command.

    Raises:
      HttpException: If an HttpError occurs.
    """
    if flags.ShouldUseGen2():
      return self._RunV2(args)

    if flags.ShouldUseGen1():
      return self._RunV1(args)

    client = client_v2.FunctionsClient(self.ReleaseTrack())
    self._v2_function = client.GetFunction(
        args.CONCEPTS.name.Parse().RelativeName()
    )

    if self._v2_function and str(self._v2_function.environment) == 'GEN_2':
      return self._RunV2(args)

    return self._RunV1(args)


def FormatTimestamp(timestamp):
  """Formats a timestamp which will be presented to a user.

  Args:
    timestamp: Raw timestamp string in RFC3339 UTC "Zulu" format.

  Returns:
    Formatted timestamp string.
  """
  return re.sub(r'(\.\d{3})\d*Z$', r'\1', timestamp.replace('T', ' '))



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""utilities function for partner metadata."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import json

from apitools.base.protorpclite import protojson
from apitools.base.py import encoding
from apitools.base.py import extra_types
from googlecloudsdk.api_lib.compute import exceptions
from googlecloudsdk.api_lib.util import apis as core_apis
from googlecloudsdk.calliope import arg_parsers

messages = core_apis.GetMessagesModule('compute', 'alpha')


class NullValueInAddPartnerMetadataException(exceptions.Error):
  """Null values not allowed in add-partner-metadata."""


def AddPartnerMetadataArgs(parser):
  """Adds --metadata and --metadata-from-file flags."""
  parser.add_argument(
      '--partner-metadata',
      type=arg_parsers.ArgDict(),
      help=(
          'Partner Metadata assigned to the instance. A map from a'
          ' subdomain(namespace) to entries map.'
      ),
      default={},
      metavar='KEY=VALUE',
      action=arg_parsers.UpdateAction,
  )
  parser.add_argument(
      '--partner-metadata-from-file',
      type=arg_parsers.FileContents(),
      help=(
          'path to json local file which including the definintion of partner'
          ' metadata.'
      ),
      metavar='LOCAL_FILE_PATH',
  )


def CreatePartnerMetadataDict(args):
  """create partner metadata from the given args.

  Args:
    args: args containing partner-metadata or partner-metadata-from-file flags

  Returns:
    python dict contains partner metadata from given args.
  """
  return _CreatePartnerMetadataDict(
      args.partner_metadata, args.partner_metadata_from_file
  )


def _CreatePartnerMetadataDict(
    partner_metadata, partner_metadata_from_file=None
):
  """create partner metadata from the given args.

  Args:
    partner_metadata: partner metadata dictionary.
    partner_metadata_from_file: partner metadata file content.

  Returns:
    python dict contains partner metadata from given args.
  """
  partner_metadata_file = {}
  if partner_metadata_from_file:
    partner_metadata_file = json.loads(partner_metadata_from_file)
  partner_metadata_dict = {}
  for key in partner_metadata_file.keys():
    if 'entries' in partner_metadata_file[key]:
      partner_metadata_dict[key] = partner_metadata_file[key]
    else:
      partner_metadata_dict[key] = {'entries': partner_metadata_file[key]}
  for key, value in partner_metadata.items():
    namespace, entry = key.split('/')
    if namespace not in partner_metadata_dict:
      partner_metadata_dict[namespace] = {'entries': {}}
    partner_metadata_dict[namespace]['entries'][entry] = json.loads(value)
  return partner_metadata_dict


def ValidatePartnerMetadata(partner_metadata):
  for key in partner_metadata.keys():
    if partner_metadata[key] is None:
      raise NullValueInAddPartnerMetadataException(
          'Null values are not allowed in partner metadata.'
      )
    if isinstance(partner_metadata[key], dict):
      ValidatePartnerMetadata(partner_metadata[key])


def ConvertStructuredEntries(structured_entries):
  structured_entries_message = messages.StructuredEntries()
  if structured_entries is None:
    return structured_entries_message
  structured_entries_message.entries = messages.StructuredEntries.EntriesValue()
  for key, value in structured_entries['entries'].items():
    structured_entries_message.entries.additionalProperties.append(
        messages.StructuredEntries.EntriesValue.AdditionalProperty(
            key=key, value=encoding.DictToMessage(value, extra_types.JsonValue)
        )
    )
  return structured_entries_message


def ConvertPartnerMetadataDictToMessage(partner_metadata_dict):
  partner_metadata_message = messages.PartnerMetadata.PartnerMetadataValue()
  for namespace, structured_entries in partner_metadata_dict.items():
    partner_metadata_message.additionalProperties.append(
        messages.PartnerMetadata.PartnerMetadataValue.AdditionalProperty(
            key=namespace, value=ConvertStructuredEntries(structured_entries)
        )
    )
  return partner_metadata_message


def ConvertStructuredEntriesToJson(structured_entries_message):
  structured_entries_dict = {'entries': {}}
  for (
      structured_entry
  ) in structured_entries_message.entries.additionalProperties:
    structured_entries_dict['entries'][structured_entry.key] = (
        encoding.MessageToDict(structured_entry.value)
    )
  return json.dumps(structured_entries_dict)


def EncodeStructuredEntries(structured_entries_message):
  if structured_entries_message.entries is None:
    return 'null'
  return ConvertStructuredEntriesToJson(structured_entries_message)


def DecodeStructuredEntries(structured_entries):
  return protojson.ProtoJson().decode_message(
      messages.StructuredEntries, structured_entries
  )

if hasattr(messages, 'StructuredEntries'):
  encoding.RegisterCustomMessageCodec(
      encoder=EncodeStructuredEntries, decoder=DecodeStructuredEntries
  )(messages.StructuredEntries)



"""Generated client library for serviceusage version v2."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.serviceusage.v2 import serviceusage_v2_messages as messages


class ServiceusageV2(base_api.BaseApiClient):
  """Generated client library for service serviceusage version v2."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://serviceusage.googleapis.com/'
  MTLS_BASE_URL = 'https://serviceusage.mtls.googleapis.com/'

  _PACKAGE = 'serviceusage'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform', 'https://www.googleapis.com/auth/cloud-platform.read-only', 'https://www.googleapis.com/auth/service.management']
  _VERSION = 'v2'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'ServiceusageV2'
  _URL_VERSION = 'v2'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new serviceusage handle."""
    url = url or self.BASE_URL
    super(ServiceusageV2, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.consumerPolicies = self.ConsumerPoliciesService(self)
    self.groups_publicValues = self.GroupsPublicValuesService(self)
    self.groups_values = self.GroupsValuesService(self)
    self.groups = self.GroupsService(self)
    self.operations = self.OperationsService(self)
    self.publicServiceApis = self.PublicServiceApisService(self)
    self.publicValueInfo = self.PublicValueInfoService(self)
    self.services_dependencies = self.ServicesDependenciesService(self)
    self.services_groups_flattenedMembers = self.ServicesGroupsFlattenedMembersService(self)
    self.services_groups_members = self.ServicesGroupsMembersService(self)
    self.services_groups = self.ServicesGroupsService(self)
    self.services = self.ServicesService(self)
    self.v2 = self.V2Service(self)

  class ConsumerPoliciesService(base_api.BaseApiService):
    """Service class for the consumerPolicies resource."""

    _NAME = 'consumerPolicies'

    def __init__(self, client):
      super(ServiceusageV2.ConsumerPoliciesService, self).__init__(client)
      self._upload_configs = {
          }

    def AddEnableRules(self, request, global_params=None):
      r"""Unimplemented. Do not use. Add enable rules to a resource's consumer policy.

      Args:
        request: (ServiceusageConsumerPoliciesAddEnableRulesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('AddEnableRules')
      return self._RunMethod(
          config, request, global_params=global_params)

    AddEnableRules.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/consumerPolicies/{consumerPoliciesId}:addEnableRules',
        http_method='POST',
        method_id='serviceusage.consumerPolicies.addEnableRules',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v2/{+parent}:addEnableRules',
        request_field='addEnableRulesRequest',
        request_type_name='ServiceusageConsumerPoliciesAddEnableRulesRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Unimplemented. Do not use. Get consumer policy for a resource.

      Args:
        request: (ServiceusageConsumerPoliciesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ConsumerPolicy) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/consumerPolicies/{consumerPoliciesId}',
        http_method='GET',
        method_id='serviceusage.consumerPolicies.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='ServiceusageConsumerPoliciesGetRequest',
        response_type_name='ConsumerPolicy',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Unimplemented. Do not use. Update consumer policy for a resource.

      Args:
        request: (ServiceusageConsumerPoliciesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/consumerPolicies/{consumerPoliciesId}',
        http_method='PATCH',
        method_id='serviceusage.consumerPolicies.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v2/{+name}',
        request_field='consumerPolicy',
        request_type_name='ServiceusageConsumerPoliciesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def RemoveEnableRules(self, request, global_params=None):
      r"""Unimplemented. Do not use. Remove enable rules from a resource's consumer policy.

      Args:
        request: (ServiceusageConsumerPoliciesRemoveEnableRulesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('RemoveEnableRules')
      return self._RunMethod(
          config, request, global_params=global_params)

    RemoveEnableRules.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/consumerPolicies/{consumerPoliciesId}:removeEnableRules',
        http_method='POST',
        method_id='serviceusage.consumerPolicies.removeEnableRules',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v2/{+parent}:removeEnableRules',
        request_field='removeEnableRulesRequest',
        request_type_name='ServiceusageConsumerPoliciesRemoveEnableRulesRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class GroupsPublicValuesService(base_api.BaseApiService):
    """Service class for the groups_publicValues resource."""

    _NAME = 'groups_publicValues'

    def __init__(self, client):
      super(ServiceusageV2.GroupsPublicValuesService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""Unimplemented. Do not use. List public values in a service group. Currently, we only support two groups "googleServices" and "partnerServices" and return public services in these two groups.

      Args:
        request: (ServiceusageGroupsPublicValuesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListPublicValuesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/groups/{groupsId}/publicValues',
        http_method='GET',
        method_id='serviceusage.groups.publicValues.list',
        ordered_params=['group'],
        path_params=['group'],
        query_params=['pageSize', 'pageToken', 'parent'],
        relative_path='v2/{+group}/publicValues',
        request_field='',
        request_type_name='ServiceusageGroupsPublicValuesListRequest',
        response_type_name='ListPublicValuesResponse',
        supports_download=False,
    )

  class GroupsValuesService(base_api.BaseApiService):
    """Service class for the groups_values resource."""

    _NAME = 'groups_values'

    def __init__(self, client):
      super(ServiceusageV2.GroupsValuesService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""Unimplemented. Do not use. List values in a service group. Currently, we only support two groups "partnerServices" and "googleServices" and return services in these two groups.

      Args:
        request: (ServiceusageGroupsValuesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListValuesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/groups/{groupsId}/values',
        http_method='GET',
        method_id='serviceusage.groups.values.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/values',
        request_field='',
        request_type_name='ServiceusageGroupsValuesListRequest',
        response_type_name='ListValuesResponse',
        supports_download=False,
    )

  class GroupsService(base_api.BaseApiService):
    """Service class for the groups resource."""

    _NAME = 'groups'

    def __init__(self, client):
      super(ServiceusageV2.GroupsService, self).__init__(client)
      self._upload_configs = {
          }

  class OperationsService(base_api.BaseApiService):
    """Service class for the operations resource."""

    _NAME = 'operations'

    def __init__(self, client):
      super(ServiceusageV2.OperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (ServiceusageOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/operations/{operationsId}',
        http_method='GET',
        method_id='serviceusage.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='ServiceusageOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (ServiceusageOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        http_method='GET',
        method_id='serviceusage.operations.list',
        ordered_params=[],
        path_params=[],
        query_params=['filter', 'name', 'pageSize', 'pageToken'],
        relative_path='v2/operations',
        request_field='',
        request_type_name='ServiceusageOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class PublicServiceApisService(base_api.BaseApiService):
    """Service class for the publicServiceApis resource."""

    _NAME = 'publicServiceApis'

    def __init__(self, client):
      super(ServiceusageV2.PublicServiceApisService, self).__init__(client)
      self._upload_configs = {
          }

    def Fetch(self, request, global_params=None):
      r"""Unimplemented. Do not use. Fetch the information of public service APIs.

      Args:
        request: (FetchPublicServiceApisRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FetchPublicServiceApisResponse) The response message.
      """
      config = self.GetMethodConfig('Fetch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Fetch.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='serviceusage.publicServiceApis.fetch',
        ordered_params=[],
        path_params=[],
        query_params=[],
        relative_path='v2/publicServiceApis:fetch',
        request_field='<request>',
        request_type_name='FetchPublicServiceApisRequest',
        response_type_name='FetchPublicServiceApisResponse',
        supports_download=False,
    )

  class PublicValueInfoService(base_api.BaseApiService):
    """Service class for the publicValueInfo resource."""

    _NAME = 'publicValueInfo'

    def __init__(self, client):
      super(ServiceusageV2.PublicValueInfoService, self).__init__(client)
      self._upload_configs = {
          }

    def Fetch(self, request, global_params=None):
      r"""Unimplemented. Do not use. Fetch the information of public values.

      Args:
        request: (FetchPublicValueInfoRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FetchPublicValueInfoResponse) The response message.
      """
      config = self.GetMethodConfig('Fetch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Fetch.method_config = lambda: base_api.ApiMethodInfo(
        http_method='POST',
        method_id='serviceusage.publicValueInfo.fetch',
        ordered_params=[],
        path_params=[],
        query_params=[],
        relative_path='v2/publicValueInfo:fetch',
        request_field='<request>',
        request_type_name='FetchPublicValueInfoRequest',
        response_type_name='FetchPublicValueInfoResponse',
        supports_download=False,
    )

  class ServicesDependenciesService(base_api.BaseApiService):
    """Service class for the services_dependencies resource."""

    _NAME = 'services_dependencies'

    def __init__(self, client):
      super(ServiceusageV2.ServicesDependenciesService, self).__init__(client)
      self._upload_configs = {
          }

    def GetClosure(self, request, global_params=None):
      r"""Unimplemented. Do not use. Get closure of a service dependency group. TODO (hanshao): remove this API once front-end migrates to ListFlattenedMembers.

      Args:
        request: (ServiceusageServicesDependenciesGetClosureRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DependencyClosure) The response message.
      """
      config = self.GetMethodConfig('GetClosure')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetClosure.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/services/{servicesId}/dependencies/{dependenciesId}/closure',
        http_method='GET',
        method_id='serviceusage.services.dependencies.getClosure',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='ServiceusageServicesDependenciesGetClosureRequest',
        response_type_name='DependencyClosure',
        supports_download=False,
    )

  class ServicesGroupsFlattenedMembersService(base_api.BaseApiService):
    """Service class for the services_groups_flattenedMembers resource."""

    _NAME = 'services_groups_flattenedMembers'

    def __init__(self, client):
      super(ServiceusageV2.ServicesGroupsFlattenedMembersService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""Unimplemented. Do not use. List flattened members for the given service group.

      Args:
        request: (ServiceusageServicesGroupsFlattenedMembersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListFlattenedMembersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/services/{servicesId}/groups/{groupsId}/flattenedMembers',
        http_method='GET',
        method_id='serviceusage.services.groups.flattenedMembers.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/flattenedMembers',
        request_field='',
        request_type_name='ServiceusageServicesGroupsFlattenedMembersListRequest',
        response_type_name='ListFlattenedMembersResponse',
        supports_download=False,
    )

  class ServicesGroupsMembersService(base_api.BaseApiService):
    """Service class for the services_groups_members resource."""

    _NAME = 'services_groups_members'

    def __init__(self, client):
      super(ServiceusageV2.ServicesGroupsMembersService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""Unimplemented. Do not use. List members for the given service group.

      Args:
        request: (ServiceusageServicesGroupsMembersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListGroupMembersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/services/{servicesId}/groups/{groupsId}/members',
        http_method='GET',
        method_id='serviceusage.services.groups.members.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/members',
        request_field='',
        request_type_name='ServiceusageServicesGroupsMembersListRequest',
        response_type_name='ListGroupMembersResponse',
        supports_download=False,
    )

  class ServicesGroupsService(base_api.BaseApiService):
    """Service class for the services_groups resource."""

    _NAME = 'services_groups'

    def __init__(self, client):
      super(ServiceusageV2.ServicesGroupsService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""Unimplemented. Do not use. List service groups owned by the given service.

      Args:
        request: (ServiceusageServicesGroupsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListServiceGroupsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/services/{servicesId}/groups',
        http_method='GET',
        method_id='serviceusage.services.groups.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2/{+parent}/groups',
        request_field='',
        request_type_name='ServiceusageServicesGroupsListRequest',
        response_type_name='ListServiceGroupsResponse',
        supports_download=False,
    )

  class ServicesService(base_api.BaseApiService):
    """Service class for the services resource."""

    _NAME = 'services'

    def __init__(self, client):
      super(ServiceusageV2.ServicesService, self).__init__(client)
      self._upload_configs = {
          }

    def GetReverseClosure(self, request, global_params=None):
      r"""Unimplemented. Do not use. Get reverse dependency closure of a single service.

      Args:
        request: (ServiceusageServicesGetReverseClosureRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ReverseDependencyClosure) The response message.
      """
      config = self.GetMethodConfig('GetReverseClosure')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetReverseClosure.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/services/{servicesId}/reverseClosure',
        http_method='GET',
        method_id='serviceusage.services.getReverseClosure',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='ServiceusageServicesGetReverseClosureRequest',
        response_type_name='ReverseDependencyClosure',
        supports_download=False,
    )

  class V2Service(base_api.BaseApiService):
    """Service class for the v2 resource."""

    _NAME = 'v2'

    def __init__(self, client):
      super(ServiceusageV2.V2Service, self).__init__(client)
      self._upload_configs = {
          }

    def CheckValue(self, request, global_params=None):
      r""" Unimplemented. Do not use. Tests a value against the result of merging consumer policies in the resource hierarchy. This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking.

      Args:
        request: (ServiceusageCheckValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (CheckValueResponse) The response message.
      """
      config = self.GetMethodConfig('CheckValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    CheckValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}:checkValue',
        http_method='POST',
        method_id='serviceusage.checkValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:checkValue',
        request_field='checkValueRequest',
        request_type_name='ServiceusageCheckValueRequest',
        response_type_name='CheckValueResponse',
        supports_download=False,
    )

    def FetchServiceApis(self, request, global_params=None):
      r"""Unimplemented. Do not use. Fetch the information of service APIs.

      Args:
        request: (ServiceusageFetchServiceApisRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FetchServiceApisResponse) The response message.
      """
      config = self.GetMethodConfig('FetchServiceApis')
      return self._RunMethod(
          config, request, global_params=global_params)

    FetchServiceApis.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}:fetchServiceApis',
        http_method='POST',
        method_id='serviceusage.fetchServiceApis',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:fetchServiceApis',
        request_field='fetchServiceApisRequest',
        request_type_name='ServiceusageFetchServiceApisRequest',
        response_type_name='FetchServiceApisResponse',
        supports_download=False,
    )

    def FetchValueInfo(self, request, global_params=None):
      r"""Unimplemented. Do not use. Fetch the information of values.

      Args:
        request: (ServiceusageFetchValueInfoRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FetchValueInfoResponse) The response message.
      """
      config = self.GetMethodConfig('FetchValueInfo')
      return self._RunMethod(
          config, request, global_params=global_params)

    FetchValueInfo.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}:fetchValueInfo',
        http_method='POST',
        method_id='serviceusage.fetchValueInfo',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}:fetchValueInfo',
        request_field='fetchValueInfoRequest',
        request_type_name='ServiceusageFetchValueInfoRequest',
        response_type_name='FetchValueInfoResponse',
        supports_download=False,
    )

    def GetEffectivePolicy(self, request, global_params=None):
      r"""Unimplemented. Do not use. Get effective consumer policy for a resource, which contains enabled value information of consumer policies from the resource hierarchy.

      Args:
        request: (ServiceusageGetEffectivePolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (EffectivePolicy) The response message.
      """
      config = self.GetMethodConfig('GetEffectivePolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetEffectivePolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/effectivePolicy',
        http_method='GET',
        method_id='serviceusage.getEffectivePolicy',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2/{+name}',
        request_field='',
        request_type_name='ServiceusageGetEffectivePolicyRequest',
        response_type_name='EffectivePolicy',
        supports_download=False,
    )

    def UpdateConsumerPolicy(self, request, global_params=None):
      r"""Unimplemented. Do not use. Update consumer policy for a resource and returns LRO.

      Args:
        request: (ServiceusageUpdateConsumerPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('UpdateConsumerPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateConsumerPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2/{v2Id}/{v2Id1}/consumerPolicy',
        http_method='PATCH',
        method_id='serviceusage.updateConsumerPolicy',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v2/{+name}',
        request_field='consumerPolicy',
        request_type_name='ServiceusageUpdateConsumerPolicyRequest',
        response_type_name='Operation',
        supports_download=False,
    )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""services policies add-enable-rule command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import collections

from googlecloudsdk.api_lib.services import serviceusage
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.services import common_flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties

_PROJECT_RESOURCE = 'projects/{}'
_FOLDER_RESOURCE = 'folders/{}'
_ORGANIZATION_RESOURCE = 'organizations/{}'
_CONSUMER_POLICY_DEFAULT = '/consumerPolicies/{}'

_OP_BASE_CMD = 'gcloud beta services operations '
_OP_WAIT_CMD = _OP_BASE_CMD + 'wait {0}'

_DETAILED_HELP = {
    'DESCRIPTION': """\
        Add service(s) to a consumer policy for a project, folder or organization.
        """,
    'EXAMPLES': """\
        To add enable rule for a service called `my-consumed-service` on default consumer policy on the current
        project, run:

          $ {command} my-consumed-service
              OR
          $ {command} my-consumed-service --policy_name=default

        To add enable rule for a service called `my-consumed-service` on default consumer policy on the
        project `my-project` , run:

          $ {command} my-consumed-service --project=my-project
              OR
          $ {command} my-consumed-service --policy_name=default --project=my-project

        To run the same command asynchronously (non-blocking), run:

          $ {command} my-consumed-service --async

        To enable services called `service1`, `service2`, and `service3` on the
        current project default policy , run:

          $ {command} service1 service2 service3
        """,
}


# TODO(b/274633761) make command public after preview.
@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class AddEnableRules(base.SilentCommand):
  """Add service(s) to a consumer policy for a project, folder or organization."""

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    common_flags.available_service_flag(
        suffix='to add enable rule for'
    ).AddToParser(parser)
    parser.add_argument(
        '--policy-name',
        help=(
            'Name of the consumer policy. Currently only "default" is'
            ' supported.'
        ),
        default='default',
    )
    common_flags.add_resource_args(parser)
    base.ASYNC_FLAG.AddToParser(parser)

    parser.display_info.AddFormat("""
        table(
            services:label=''
        )
      """)

  def Run(self, args):
    """Run services policies add-enable-rule.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      Nothing.
    """
    if args.IsSpecified('project'):
      project = args.project
    else:
      project = properties.VALUES.core.project.Get(required=True)
    resource_name = _PROJECT_RESOURCE.format(project)
    if args.IsSpecified('folder'):
      folder = args.folder
      resource_name = _FOLDER_RESOURCE.format(folder)
    else:
      folder = None
    if args.IsSpecified('organization'):
      organization = args.organization
      resource_name = _ORGANIZATION_RESOURCE.format(organization)
    else:
      organization = None

    op = serviceusage.AddEnableRule(
        args.service,
        project,
        args.policy_name,
        folder,
        organization,
    )
    if args.async_:
      cmd = _OP_WAIT_CMD.format(op.name)
      log.status.Print(
          'Asynchronous operation is in progress... '
          'Use the following command to wait for its '
          'completion:\n {0}'.format(cmd)
      )

    update_policy = serviceusage.GetConsumerPolicy(
        resource_name + _CONSUMER_POLICY_DEFAULT.format(args.policy_name)
    )

    resources = collections.namedtuple('Values', ['services'])
    result = []
    for value in update_policy.enableRules[0].values:
      result.append(resources(value))
    log.status.Print(
        'Consumer policy ('
        + resource_name
        + _CONSUMER_POLICY_DEFAULT.format(args.policy_name)
        + ') has been updated to:',
    )
    return result


AddEnableRules.detailed_help = _DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for GEC cluster commands."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import exceptions as gcloud_exceptions
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.core import properties


def SetAdminUsers(messages, args, request):
  """Sets the cluster.authorization.admin_users to the user if unspecified.

  Args:
    messages: message module of edgecontainer cluster.
    args: command line arguments.
    request: API request to be issued
  """

  request.cluster.authorization = messages.Authorization()
  request.cluster.authorization.adminUsers = messages.ClusterUser()
  if flags.FlagIsExplicitlySet(args, 'admin_users'):
    request.cluster.authorization.adminUsers.username = args.admin_users
    return

  if properties.VALUES.auth.credential_file_override.Get() is not None:
    raise gcloud_exceptions.RequiredArgumentException(
        '--admin-users', 'Required if auth/credential_file_override is defined.'
    )

  service_account_override = (
      properties.VALUES.auth.impersonate_service_account.Get()
  )
  if service_account_override is not None:
    request.cluster.authorization.adminUsers.username = service_account_override
  else:
    default_account = properties.VALUES.core.account.Get()
    if default_account is None:
      raise gcloud_exceptions.RequiredArgumentException(
          '--admin-users',
          (
              'Required if no account is active and'
              ' --impersonate-service-account is undefined.'
          ),
      )
    request.cluster.authorization.adminUsers.username = default_account



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing backend services."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.backend_services import flags
from googlecloudsdk.core import properties

_DETAILED_HELP = {
    "DESCRIPTION": """\
          $ {command}  \
          retrieves the list of backend service resources in the
          specified project for which you have compute.backendService.get
          and compute.backendService.use permissions. This command is useful
          when you're creating load balancers in a Shared VPC environment
          and you want to use [cross-project service
          referencing](https://cloud.google.com/load-balancing/docs/https#cross-project).
          You can use this command to find out which backend
          services in other projects are available to you for referencing.
        """,
    "EXAMPLES": """\
        To list all global backend services in a project, run:

          $ {command} --global

        To list all backend services in a region, run:

          $ {command} --region=REGION
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class ListUsable(base.ListCommand):
  """List usable backend services."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.GLOBAL_REGIONAL_BACKEND_SERVICE_NOT_REQUIRED_ARG.AddArgument(parser)
    parser.display_info.AddFormat(flags.DEFAULT_BETA_LIST_FORMAT)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    messages = holder.client.messages

    resource_scope, scope_value = (
        flags.GLOBAL_REGIONAL_BACKEND_SERVICE_NOT_REQUIRED_ARG.scopes.SpecifiedByArgs(
            args
        )
    )

    if resource_scope.scope_enum == compute_scope.ScopeEnum.GLOBAL:
      request = messages.ComputeBackendServicesListUsableRequest(
          project=properties.VALUES.core.project.Get(required=True)
      )
      apitools = client.apitools_client.backendServices
    elif resource_scope.scope_enum == compute_scope.ScopeEnum.REGION:
      request = messages.ComputeRegionBackendServicesListUsableRequest(
          region=scope_value,
          project=properties.VALUES.core.project.Get(required=True),
      )
      apitools = client.apitools_client.regionBackendServices

    return list_pager.YieldFromList(
        apitools,
        request,
        method="ListUsable",
        batch_size_attribute="maxResults",
        batch_size=500,
        field="items",
    )



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for adding an interface to a Compute Engine router."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import encoding
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_errors
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.interconnects.attachments import (
    flags as attachment_flags,
)
from googlecloudsdk.command_lib.compute.networks.subnets import flags as subnet_flags
from googlecloudsdk.command_lib.compute.routers import flags as router_flags
from googlecloudsdk.command_lib.compute.vpn_tunnels import (
    flags as vpn_tunnel_flags,
)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class AddInterface(base.UpdateCommand):
  """Add an interface to a Compute Engine router.

  *{command}* is used to add an interface to a Compute Engine
  router.
  """

  ROUTER_ARG = None
  VPN_TUNNEL_ARG = None
  INTERCONNECT_ATTACHMENT_ARG = None
  SUBNETWORK_ARG = None

  @classmethod
  def _Args(cls, parser, enable_ipv6_bgp=False):
    cls.ROUTER_ARG = router_flags.RouterArgument()
    cls.ROUTER_ARG.AddArgument(parser, operation_type='update')

    link_parser = parser.add_mutually_exclusive_group(required=True)

    vpn_tunnel_group = link_parser.add_argument_group('VPN Tunnel')
    cls.VPN_TUNNEL_ARG = vpn_tunnel_flags.VpnTunnelArgumentForRouter(
        required=False
    )
    cls.VPN_TUNNEL_ARG.AddArgument(vpn_tunnel_group)

    interconnect_attachment_group = link_parser.add_argument_group(
        'Interconnect Attachment'
    )
    cls.INTERCONNECT_ATTACHMENT_ARG = (
        attachment_flags.InterconnectAttachmentArgumentForRouter()
    )
    cls.INTERCONNECT_ATTACHMENT_ARG.AddArgument(interconnect_attachment_group)

    subnetwork_group = link_parser.add_argument_group('Subnetwork')
    cls.SUBNETWORK_ARG = subnet_flags.SubnetworkArgumentForRouter()
    cls.SUBNETWORK_ARG.AddArgument(subnetwork_group)

    router_flags.AddInterfaceArgs(parser, enable_ipv6_bgp=enable_ipv6_bgp)

  @classmethod
  def Args(cls, parser):
    cls._Args(parser)

  def _GetGetRequest(self, client, router_ref):
    return (
        client.apitools_client.routers,
        'Get',
        client.messages.ComputeRoutersGetRequest(
            router=router_ref.Name(),
            region=router_ref.region,
            project=router_ref.project,
        ),
    )

  def _GetSetRequest(self, client, router_ref, replacement):
    return (
        client.apitools_client.routers,
        'Patch',
        client.messages.ComputeRoutersPatchRequest(
            router=router_ref.Name(),
            routerResource=replacement,
            region=router_ref.region,
            project=router_ref.project,
        ),
    )

  def Modify(self, client, resources, args, existing, enable_ipv6_bgp=False):
    replacement = encoding.CopyProtoMessage(existing)
    mask = None
    interface_name = args.interface_name

    if args.ip_address is not None:
      if args.subnetwork is None and args.mask_length is not None:
        mask = '{0}/{1}'.format(args.ip_address, args.mask_length)
      elif args.subnetwork is None:
        raise parser_errors.ArgumentException(
            '--mask-length must be set if --ip-address is set'
        )
      elif args.mask_length is not None:
        raise parser_errors.ArgumentException(
            '--mask-length cannot be set if --subnetwork is set'
        )

    ip_version = None
    if enable_ipv6_bgp and args.ip_version is not None:
      ip_version = client.messages.RouterInterface.IpVersionValueValuesEnum(
          args.ip_version
      )

    if not args.vpn_tunnel_region:
      args.vpn_tunnel_region = replacement.region
    vpn_ref = None
    if args.vpn_tunnel is not None:
      vpn_ref = self.VPN_TUNNEL_ARG.ResolveAsResource(
          args,
          resources,
          scope_lister=compute_flags.GetDefaultScopeLister(client),
      )

    if not args.interconnect_attachment_region:
      args.interconnect_attachment_region = replacement.region
    attachment_ref = None
    if args.interconnect_attachment is not None:
      attachment_ref = self.INTERCONNECT_ATTACHMENT_ARG.ResolveAsResource(
          args, resources
      )

    subnetwork_ref = None
    private_ip_address = None
    redundant_interface = None
    if args.subnetwork is not None:
      subnetwork_ref = self.SUBNETWORK_ARG.ResolveAsResource(args, resources)
      private_ip_address = args.ip_address
      redundant_interface = args.redundant_interface

    interface = client.messages.RouterInterface(
        name=interface_name,
        linkedVpnTunnel=(vpn_ref.SelfLink() if vpn_ref else None),
        linkedInterconnectAttachment=(
            attachment_ref.SelfLink() if attachment_ref else None
        ),
        subnetwork=(subnetwork_ref.SelfLink() if subnetwork_ref else None),
        ipRange=mask,
        privateIpAddress=private_ip_address,
        redundantInterface=redundant_interface,
    )
    if ip_version is not None:
      interface.ipVersion = ip_version

    replacement.interfaces.append(interface)

    return replacement

  def _Run(self, args, enable_ipv6_bgp=False):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    router_ref = self.ROUTER_ARG.ResolveAsResource(args, holder.resources)
    get_request = self._GetGetRequest(client, router_ref)

    objects = client.MakeRequests([get_request])

    new_object = self.Modify(
        client,
        holder.resources,
        args,
        objects[0],
        enable_ipv6_bgp=enable_ipv6_bgp,
    )

    return client.MakeRequests(
        [self._GetSetRequest(client, router_ref, new_object)]
    )

  def Run(self, args):
    return self._Run(args)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class AddInterfaceBeta(AddInterface):
  """Add an interface to a Compute Engine router.

  *{command}* is used to add an interface to a Compute Engine
  router.
  """
  pass


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class AddInterfaceAlpha(AddInterfaceBeta):
  """Add an interface to a Compute Engine router.

  *{command}* is used to add an interface to a Compute Engine
  router.
  """

  @classmethod
  def Args(cls, parser):
    cls._Args(parser, enable_ipv6_bgp=True)

  def Run(self, args):
    return self._Run(args, enable_ipv6_bgp=True)



# -*- coding: utf-8 -*- # # Copyright 2020 Google LLC. All Rights Reserved.
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Google Cloud Identity Sources client."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.vmware import util
from googlecloudsdk.command_lib.util.apis import arg_utils


class IdentitySourcesClient(util.VmwareClientBase):
  """Identity Sources client."""

  def __init__(self):
    super(IdentitySourcesClient, self).__init__()
    self.service = self.client.projects_locations_privateClouds_identitySources

  def Create(
      self,
      resource,
      domain,
      base_users_dn,
      base_groups_dn,
      domain_user,
      domain_password,
      protocol,
      primary_server,
      domain_alias=None,
      secondary_server=None,
      ssl_certificates=None,
  ):
    protocol_enum_value = arg_utils.ChoiceEnumMapper(
        arg_name='protocol',
        message_enum=self.messages.IdentitySource.ProtocolValueValuesEnum,
        include_filter=lambda x: 'PROTOCOL_UNSPECIFIED' not in x,
    ).GetEnumForChoice(arg_utils.EnumNameToChoice(protocol))

    source = self.messages.IdentitySource(
        domain=domain,
        baseUsersDn=base_users_dn,
        baseGroupsDn=base_groups_dn,
        domainUser=domain_user,
        domainPassword=domain_password,
        protocol=protocol_enum_value,
        specificDomainControllers=self.messages.SpecificDomainControllers(
            primaryServerUri=primary_server,
            secondaryServerUri=secondary_server,
        ),
        domainAlias=domain_alias,
        sslCertificates=ssl_certificates,
        applianceType=self.messages.IdentitySource.ApplianceTypeValueValuesEnum.VCENTER,
    )

    request = self.messages.VmwareengineProjectsLocationsPrivateCloudsIdentitySourcesCreateRequest(
        identitySource=source,
        identitySourceId=resource.Name(),
        parent=resource.Parent().RelativeName(),
    )

    return self.service.Create(request)

  def Get(self, resource):
    request = self.messages.VmwareengineProjectsLocationsPrivateCloudsIdentitySourcesGetRequest(
        name=resource.RelativeName()
    )
    return self.service.Get(request)

  def List(self, resource):
    address_name = resource.RelativeName()
    request = self.messages.VmwareengineProjectsLocationsPrivateCloudsIdentitySourcesListRequest(
        parent=address_name
    )

    return list_pager.YieldFromList(
        self.service,
        request,
        batch_size_attribute='pageSize',
        field='identitySources',
    )

  def Update(
      self,
      resource,
      base_users_dn=None,
      base_groups_dn=None,
      domain_user=None,
      domain_password=None,
      ssl_certificates=None,
  ):
    source = self.messages.IdentitySource()
    update_mask = []

    if base_users_dn is not None:
      source.baseUsersDn = base_users_dn
      update_mask.append('base_users_dn')
    if base_groups_dn is not None:
      source.baseGroupsDn = base_groups_dn
      update_mask.append('base_groups_dn')
    if domain_user is not None:
      source.domainUser = domain_user
      update_mask.append('domain_user')
    if domain_password is not None:
      source.domainPassword = domain_password
      update_mask.append('domain_password')
    if ssl_certificates:
      source.sslCertificates = ssl_certificates
      update_mask.append('ssl_certificates')

    request = self.messages.VmwareengineProjectsLocationsPrivateCloudsIdentitySourcesPatchRequest(
        identitySource=source,
        name=resource.RelativeName(),
        updateMask=','.join(update_mask),
    )

    return self.service.Patch(request)

  def Delete(self, resource):
    request = self.messages.VmwareengineProjectsLocationsPrivateCloudsIdentitySourcesDeleteRequest(
        name=resource.RelativeName()
    )
    return self.service.Delete(request)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Smart Guardrails Recommendation Utilities."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import re

from googlecloudsdk.api_lib.recommender import insight
from googlecloudsdk.command_lib.projects import util as project_util

_PROJECT_WARNING_MESSAGE = """Shutting down this project will immediately:
  - Stop all traffic and billing.
  - Start deleting resources.
  - Schedule the final deletion of the project after 30 days.
  - Block your access to the project.
  - Notify the owner of the project.

Learn more about the shutdown process at
https://cloud.google.com/resource-manager/docs/creating-managing-projects#shutting_down_projects
"""

_PROJECT_RISK_MESSAGE = (
    "*** HIGH-RISK CHANGE WARNING ***: If you shut down this project you risk "
    "losing data or interrupting your services"
)

_PROJECT_REASONS_PREFIX = ". In the last 30 days we observed this project had"

_PROJECT_ADVICE = (
    "We recommend verifying this is the correct project to shut down.\n"
)

_SA_DELETE_APP_ENGINE_WARNING_MESSAGE = (
    "This service account lets App Engine and Cloud Functions access essential"
    " Cloud Platform services such as Datastore. Deleting this account will "
    "break any current or future App Engine applications and Cloud Functions "
    "in this project.\n\n"
    "WARNING: You CANNOT  undo this action. "
    "Do not delete this service account unless you are sure you"
    " will never use App Engine in this project."
)

_SA_DELETE_COMPUTE_ENGINE_WARNING_MESSAGE = (
    "This service account lets Compute Engine access essential Cloud Platform "
    "services such as logging and Cloud Storage. Deleting this account will "
    "prevent instances that are running as this account from accessing "
    "Cloud Platform services.\n\n"
    "WARNING: You CANNOT undo this action. "
    "If you delete this account, instances in this project will only be able "
    "to access Cloud Platform services via custom service accounts."
)

_SA_WARNING_MESSAGE = (
    "Deleting this service account (SA) will delete "
    "all associated key IDs, and will prevent the account from authenticating"
    " to any Google Cloud service API.\n\n"
    "You cannot restore or roll back this change easily. We highly recommend"
    " disabling the account first, testing for any unexpected impact, and only"
    " then deleting.\n"
)

_SA_RISK_MESSAGE = (
    "*** HIGH-RISK CHANGE WARNING ***: If you delete this SA you risk "
    "interrupting your service, as we observed it was substantially used in "
    "the last 90 days"
)

_SA_ADVICE = "We recommend verifying this is the correct account to delete.\n"

_POLICY_BINDING_DELETE_WARNING_MESSAGE = (
    "You are about to delete the role [{}].\n"
)

_POLICY_BINDING_DELETE_RISK_MESSAGE = (
    "*** HIGH-RISK CHANGE WARNING ***: If you remove the role [{}], there is "
    "a high risk that you might cause interruptions because it was used "
    "in the last 90 days"
)

_POLICY_BINDING_DELETE_ADVICE = (
    "We recommend you verify the details and replace them with less privileged"
    " roles, if necessary.\n"
)

_PROJECT_INSIGHT_TYPE = "google.resourcemanager.project.ChangeRiskInsight"

_SA_INSIGHT_TYPE = "google.iam.serviceAccount.ChangeRiskInsight"

_POLICY_BINDING_INSIGHT_TYPE = "google.iam.policy.ChangeRiskInsight"

_RECOMMENDATIONS_HOME_URL = (
    "https://console.cloud.google.com/home/recommendations"
)

_MAX_NUMBER_OF_REASONS = 3


def _GetIamPolicyBindingMatcher(member, role):
  """Returns a function that matches an insight by policy binding.

  Args:
    member: Member string to fully match.
    role: Member role to fully match.

  Returns:
    A function that matches an insight object by policy binding. The returned
    function returns true iff both the member and the role match.
  """

  def Matcher(gcloud_insight):
    matches_member = False
    matches_role = False
    for additional_property in gcloud_insight.content.additionalProperties:
      if additional_property.key == "risk":
        for p in additional_property.value.object_value.properties:
          if p.key == "usageAtRisk":
            for f in p.value.object_value.properties:
              if f.key == "iamPolicyUtilization":
                for iam_p in f.value.object_value.properties:
                  if iam_p.key == "member":
                    if iam_p.value.string_value == member:
                      matches_member = True
                  if iam_p.key == "role":
                    if iam_p.value.string_value == role:
                      matches_role = True
    return matches_member and matches_role

  return Matcher


def _GetInsightLink(gcloud_insight):
  """Returns a message with a link to the associated recommendation.

  Args:
    gcloud_insight: Insight object returned by the recommender API.

  Returns:
    A string message with a link to the associated recommendation.
  """
  return ("View the full risk assessment at: {0}/view-link/{1}").format(
      _RECOMMENDATIONS_HOME_URL, gcloud_insight.name
  )


def _GetResourceRiskReasons(gcloud_insight):
  """Extracts a list of string reasons from the resource change insight.

  Args:
    gcloud_insight: Insight object returned by the recommender API.

  Returns:
    A list of strings. If no reasons could be found, then returns empty list.
    The number of reasons is limited by _MAX_NUMBER_OF_REASONS, and the last
    reason indicates how many more reasons there are if applicable.
  """
  reasons = []
  num_reasons = 0
  last_reason = ""
  for additional_property in gcloud_insight.content.additionalProperties:
    if additional_property.key == "importance":
      for p in additional_property.value.object_value.properties:
        if p.key == "detailedReasons":
          for reason in p.value.array_value.entries:
            num_reasons += 1
            if num_reasons < _MAX_NUMBER_OF_REASONS:
              reasons.append(reason.string_value)
            elif num_reasons == _MAX_NUMBER_OF_REASONS:
              last_reason = reason.string_value
  if num_reasons > _MAX_NUMBER_OF_REASONS:
    last_reason = "{} other importance indicators.".format(
        num_reasons - _MAX_NUMBER_OF_REASONS + 1
    )
  if num_reasons >= _MAX_NUMBER_OF_REASONS:
    reasons.append(last_reason)
  return reasons


def _GetDeletionRiskMessage(gcloud_insight, risk_message, reasons_prefix=""):
  """Returns a risk message for resource deletion.

  Args:
    gcloud_insight: Insight object returned by the recommender API.
    risk_message: String risk message.
    reasons_prefix: String prefix before listing reasons.

  Returns:
    Formatted string risk message with reasons if any. The reasons are
    extracted from the gcloud_insight object.
  """
  reasons = _GetResourceRiskReasons(gcloud_insight)
  if not reasons:
    return risk_message + ".\n"
  if len(reasons) == 1:
    return "{0}{1} {2}\n".format(risk_message, reasons_prefix, reasons[0])
  message = "{0}{1}:\n".format(risk_message, reasons_prefix)
  message += "".join("  - {0}\n".format(reason) for reason in reasons)
  return message


def _GetRiskInsight(
    release_track, project_id, insight_type, request_filter=None, matcher=None
):
  """Returns the first insight fetched by the recommender API.

  Args:
    release_track: Release track of the recommender.
    project_id: Project ID.
    insight_type: String insight type.
    request_filter: Optional string filter for the recommender.
    matcher: Matcher for the insight object. None means match all.

  Returns:
    Insight object returned by the recommender API. Returns 'None' if no
    matching insights were found. Returns the first insight object that matches
    the matcher. If no matcher, returns the first insight object fetched.
  """
  client = insight.CreateClient(release_track)
  parent_name = ("projects/{0}/locations/global/insightTypes/{1}").format(
      project_id, insight_type
  )
  result = client.List(
      parent_name, page_size=100, limit=None, request_filter=request_filter
  )
  for r in result:
    if not matcher:
      return r
    if matcher(r):
      return r
  return None


def _IsDefaultAppEngineServiceAccount(email):
  """Returns true if email is used as a default App Engine Service Account.

  Args:
    email: Service Account email.

  Returns:
    Returns true if the given email is default App Engine Service Account.
    Returns false otherwise.
  """
  return re.search(
      r"^([\w:.-]+)@appspot(\.[^.]+\.iam)?\.gserviceaccount\.com", email
  )


def _IsDefaultComputeEngineServiceAccount(email, project_number):
  """Returns true if email is used as a default Compute Engine Service Account.

  Args:
    email: Service Account email.
    project_number: Project number.

  Returns:
    Returns true if the given email is a default Compue Engine Service Account.
    Returns false otherwise.
  """
  if email == "{0}@developer.gserviceaccount.com".format(project_number):
    return True
  if email == "{0}@project.gserviceaccount.com".format(project_number):
    return True
  return re.search(
      r"^[0-9]+-compute@developer(\.[^.]+\.iam)?\.gserviceaccount\.com", email
  )


def GetProjectDeletionRisk(release_track, project_id):
  """Returns a risk assesment message for project deletion.

  Args:
    release_track: Release track of the recommender.
    project_id: Project ID.

  Returns:
    String message prompt to be displayed for project deletion.
    If the project deletion is high risk, the message includes the
    Active Assist warning.
  """
  risk_insight = _GetRiskInsight(
      release_track, project_id, _PROJECT_INSIGHT_TYPE
  )
  if risk_insight:
    return "\n".join([
        _PROJECT_WARNING_MESSAGE,
        _GetDeletionRiskMessage(
            gcloud_insight=risk_insight,
            risk_message=_PROJECT_RISK_MESSAGE,
            reasons_prefix=_PROJECT_REASONS_PREFIX,
        ),
        _PROJECT_ADVICE,
        _GetInsightLink(risk_insight),
    ])
  # If there are no risks to deleting a project,
  # return a standard warning message.
  return _PROJECT_WARNING_MESSAGE


def GetServiceAccountDeletionRisk(release_track, project_id, service_account):
  """Returns a risk assesment message for service account deletion.

  Args:
    release_track: Release track of the recommender.
    project_id: String project ID.
    service_account: Service Account email ID.

  Returns:
    String Active Assist risk warning message to be displayed in
    service account deletion prompt.
  """
  project_number = project_util.GetProjectNumber(project_id)
  # If special default App Engine SA, return a special warning.
  if _IsDefaultAppEngineServiceAccount(service_account):
    return _SA_DELETE_APP_ENGINE_WARNING_MESSAGE
  # If special default Compute Engine SA, return a special warning.
  if _IsDefaultComputeEngineServiceAccount(service_account, project_number):
    return _SA_DELETE_COMPUTE_ENGINE_WARNING_MESSAGE
  target_filter = (
      "targetResources: //iam.googleapis.com/projects/{0}/serviceAccounts/{1}"
  ).format(project_number, service_account)
  risk_insight = _GetRiskInsight(
      release_track, project_id, _SA_INSIGHT_TYPE, request_filter=target_filter
  )
  if risk_insight:
    return "\n".join([
        _SA_WARNING_MESSAGE,
        _GetDeletionRiskMessage(risk_insight, _SA_RISK_MESSAGE),
        _SA_ADVICE,
        _GetInsightLink(risk_insight),
    ])
  # If there are no risks to deleting a service account,
  # return a standard warning message.
  return _SA_WARNING_MESSAGE


def GetIamPolicyBindingDeletionRisk(
    release_track, project_id, member, member_role
):
  """Returns a risk assesment message for IAM policy binding deletion.

  Args:
    release_track: Release track of the recommender.
    project_id: String project ID.
    member: IAM policy binding member.
    member_role: IAM policy binding member role.

  Returns:
    String Active Assist risk warning message to be displayed in IAM policy
    binding deletion prompt.
    If no risk exists, then returns 'None'.
  """
  # Remove prefixes like "user:", "group:", etc. from member
  # because the recommender generates a member id only.
  member = member[(member.find(":") + 1) :]
  policy_matcher = _GetIamPolicyBindingMatcher(member, member_role)
  risk_insight = _GetRiskInsight(
      release_track,
      project_id,
      _POLICY_BINDING_INSIGHT_TYPE,
      matcher=policy_matcher,
  )
  if risk_insight:
    return "\n".join([
        _POLICY_BINDING_DELETE_WARNING_MESSAGE.format(member_role),
        _GetDeletionRiskMessage(
            risk_insight,
            _POLICY_BINDING_DELETE_RISK_MESSAGE.format(member_role),
        ),
        _POLICY_BINDING_DELETE_ADVICE,
        _GetInsightLink(risk_insight),
    ])
  # If there are no risks to deleting the IAM policy binding,
  # return a standard warning message.
  return _POLICY_BINDING_DELETE_WARNING_MESSAGE.format(member_role)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility for forming Artifact Registry requests around cleanup policies."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import json

from apitools.base.py import encoding as apitools_encoding
from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.artifacts import exceptions as ar_exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.util import encoding
from googlecloudsdk.core.util import times
import six


def ParseCleanupPolicy(path):
  """Reads a cleanup policy from a JSON formatted file.

  Args:
    path: str, path to the policy file.

  Returns:
    A dict describing a cleanup policy, matching the proto description.

  Raises:
    InvalidInputValueError: The JSON file could not be parsed or the data does
    not follow the correct schema.
  """
  content = console_io.ReadFromFileOrStdin(path, binary=False)
  try:
    file_policies = json.loads(encoding.Decode(content))
  except ValueError as e:
    raise apitools_exceptions.InvalidUserInputError(
        'Could not read JSON file {}: {}'.format(path, e))
  if not isinstance(file_policies, list):
    raise apitools_exceptions.InvalidUserInputError(
        'Policy file must contain a list of policies.'
    )
  policies = dict()
  for i in range(len(file_policies)):
    policy = file_policies[i]
    if not isinstance(policy, dict):
      raise apitools_exceptions.InvalidUserInputError(
          'Invalid policy at index {}.'.format(i)
      )
    name = policy.get('name')
    if name is None:
      raise ar_exceptions.InvalidInputValueError(
          'Key "name" not found in policy.'
      )
    if name in policies:
      raise ar_exceptions.InvalidInputValueError(
          'Duplicate key "{}" in policy list.'.format(name)
      )
    action = policy.get('action')
    if action is None:
      raise ar_exceptions.InvalidInputValueError(
          'Key "action" not found in policy "{}".'.format(name)
      )
    try:
      action = action.get('type', '')
    except AttributeError as error:
      six.raise_from(
          ar_exceptions.InvalidInputValueError(
              'Invalid action "{}" in policy "{}".'.format(action, name)
          ),
          error,
      )
    condition = policy.get('condition')
    if condition is not None:
      if not isinstance(condition, dict):
        raise ar_exceptions.InvalidInputValueError(
            'Invalid value for "condition" in policy "{}".'.format(name)
        )
      for duration_key in ['versionAge', 'olderThan', 'newerThan']:
        if duration_key in condition:
          seconds = times.ParseDuration(condition[duration_key])
          condition[duration_key] = six.text_type(seconds.total_seconds) + 's'
    most_recent_versions = policy.get('mostRecentVersions')
    if 'condition' not in policy and 'mostRecentVersions' not in policy:
      raise ar_exceptions.InvalidInputValueError(
          'Key "condition" or "mostRecentVersions" not found in policy "{}".'
          .format('name')
      )
    if 'condition' in policy and 'mostRecentVersions' in policy:
      raise ar_exceptions.InvalidInputValueError(
          'Only one of "condition" or "mostRecentVersions" '
          'allowed in policy "{}".'.format(name)
      )
    policies[name] = {
        'id': name,
        'action': action,
        'condition': condition,
        'mostRecentVersions': most_recent_versions,
    }
  return policies


def SetDeleteCleanupPolicyUpdateMask(unused_ref, unused_args, request):
  """Sets update mask for deleting Cleanup Policies."""
  request.updateMask = 'cleanup_policies'
  return request


def RepositoryToCleanupPoliciesResponse(response, unused_args):
  """Formats Cleanup Policies output and displays Dry Run status."""
  if response.cleanupPolicyDryRun:
    log.status.Print('Dry run is enabled.')
  else:
    log.status.Print('Dry run is disabled.')
  if not response.cleanupPolicies:
    return []
  policies = apitools_encoding.MessageToDict(response.cleanupPolicies)
  sorted_policies = sorted(policies.values(), key=lambda p: p['id'])
  for policy in sorted_policies:
    policy['name'] = policy['id']
    del policy['id']
    policy['action'] = {'type': policy['action']}
  return sorted_policies


def DeleteCleanupPolicyFields(unused_ref, args, request):
  removed_policies = args.policynames.split(',')
  remaining_policies = []
  if request.repository.cleanupPolicies:
    for policy in request.repository.cleanupPolicies.additionalProperties:
      if policy.key not in removed_policies:
        remaining_policies.append(policy)
    request.repository.cleanupPolicies.additionalProperties = remaining_policies
  request.updateMask = None
  return request
